@@ -36,6 +36,7 @@ extern const OSSymbol *gCommand_Set_Revision_Symbol;
 extern const OSSymbol *gIOUnit_Symbol;
 extern const OSSymbol *gFirmware_Revision_Symbol;
 extern const OSSymbol *gDevice_Type_Symbol;
+extern const OSSymbol *gGUID_Symbol;
 
 const OSSymbol *gDiagnostics_Symbol;
 
@@ -248,10 +249,10 @@ IOReturn IOFireWireSBP2LUN::executeFlushAllMgmtORBs( void )
 	
 	if( fORBSetIterator )
 	{
-		IOFireWireSBP2ManagementORB * item = NULL;
-        fORBSetIterator->reset();		
+		IOFireWireSBP2ManagementORB * item = NULL;		
 		do
 		{
+            fORBSetIterator->reset();
 			item = (IOFireWireSBP2ManagementORB *)fORBSetIterator->getNextObject();
 			if( item )
 				item->release();
@@ -551,7 +552,8 @@ bool IOFireWireSBP2LUN::matchPropertyTable(OSDictionary * table)
 				compareProperty(table, gCommand_Set_Revision_Symbol) &&
 				compareProperty(table, gIOUnit_Symbol) &&
 				compareProperty(table, gFirmware_Revision_Symbol) &&
-				compareProperty(table, gDevice_Type_Symbol);
+				compareProperty(table, gDevice_Type_Symbol) &&
+                compareProperty(table, gGUID_Symbol);
 				
     return res;
 }
